@@ -20,7 +20,7 @@ from .const import (
     SCANNER_WATCHDOG_INTERVAL,
     SCANNER_WATCHDOG_TIMEOUT,
 )
-from .models import BluetoothServiceInfoBleak, HaBluetoothConnector
+from .models import BluetoothServiceInfoBleak, HaBluetoothConnector, HaScannerDetails
 
 SCANNER_WATCHDOG_INTERVAL_SECONDS: Final = SCANNER_WATCHDOG_INTERVAL.total_seconds()
 _LOGGER = logging.getLogger(__name__)
@@ -44,6 +44,7 @@ class BaseHaScanner:
         "adapter",
         "connectable",
         "connector",
+        "details",
         "name",
         "scanning",
         "source",
@@ -54,9 +55,10 @@ class BaseHaScanner:
         source: str,
         adapter: str,
         connector: HaBluetoothConnector | None = None,
+        connectable: bool = False,
     ) -> None:
         """Initialize the scanner."""
-        self.connectable = False
+        self.connectable = connectable
         self.source = source
         self.connector = connector
         self._connecting = 0
@@ -68,6 +70,12 @@ class BaseHaScanner:
         self._cancel_watchdog: asyncio.TimerHandle | None = None
         self._loop: asyncio.AbstractEventLoop | None = None
         self._manager = get_manager()
+        self.details = HaScannerDetails(
+            source=self.source,
+            connectable=self.connectable,
+            name=self.name,
+            adapter=self.adapter,
+        )
 
     def async_setup(self) -> CALLBACK_TYPE:
         """Set up the scanner."""
@@ -211,8 +219,7 @@ class BaseHaRemoteScanner(BaseHaScanner):
         connectable: bool,
     ) -> None:
         """Initialize the scanner."""
-        super().__init__(scanner_id, name, connector)
-        self.connectable = connectable
+        super().__init__(scanner_id, name, connector, connectable)
         self._details: dict[str, str | HaBluetoothConnector] = {"source": scanner_id}
         # Scanners only care about connectable devices. The manager
         # will handle taking care of availability for non-connectable devices
@@ -368,47 +375,28 @@ class BaseHaRemoteScanner(BaseHaScanner):
         """Call the registered callback."""
         self.scanning = not self._connecting
         self._last_detection = advertisement_monotonic_time
+        service_info = BluetoothServiceInfoBleak.__new__(BluetoothServiceInfoBleak)
+
         if (prev_service_info := self._previous_service_info.get(address)) is None:
             # We expect this is the rare case and since py3.11+ has
             # near zero cost try on success, and we can avoid .get()
             # which is slower than [] we use the try/except pattern.
-            device = BLEDevice(
+            service_info.device = BLEDevice(
                 address,
                 local_name,
                 {**self._details, **details},
                 rssi,  # deprecated, will be removed in newer bleak
             )
+            service_info.manufacturer_data = manufacturer_data
+            service_info.service_data = service_data
+            service_info.service_uuids = service_uuids
+            service_info.name = local_name or address
         else:
             # Merge the new data with the old data
             # to function the same as BlueZ which
             # merges the dicts on PropertiesChanged
-            prev_device = prev_service_info.device
-            prev_service_uuids = prev_service_info.service_uuids
-            prev_service_data = prev_service_info.service_data
-            prev_manufacturer_data = prev_service_info.manufacturer_data
-            prev_name = prev_device.name
-            prev_details = prev_device.details
-
-            if prev_name and (not local_name or len(prev_name) > len(local_name)):
-                local_name = prev_name
-
-            has_service_uuids = bool(service_uuids)
-            if has_service_uuids and service_uuids != prev_service_uuids:
-                service_uuids = list({*service_uuids, *prev_service_uuids})
-            elif not has_service_uuids:
-                service_uuids = prev_service_uuids
-
-            has_service_data = bool(service_data)
-            if has_service_data and service_data != prev_service_data:
-                service_data = {**prev_service_data, **service_data}
-            elif not has_service_data:
-                service_data = prev_service_data
-
-            has_manufacturer_data = bool(manufacturer_data)
-            if has_manufacturer_data and manufacturer_data != prev_manufacturer_data:
-                manufacturer_data = {**prev_manufacturer_data, **manufacturer_data}
-            elif not has_manufacturer_data:
-                manufacturer_data = prev_manufacturer_data
+            service_info.device = prev_service_info.device
+            prev_name = prev_service_info.device.name
             #
             # Bleak updates the BLEDevice via create_or_update_device.
             # We need to do the same to ensure integrations that already
@@ -417,21 +405,56 @@ class BaseHaRemoteScanner(BaseHaScanner):
             #
             # https://github.com/hbldh/bleak/blob/222618b7747f0467dbb32bd3679f8cfaa19b1668/bleak/backends/scanner.py#L203
             #
-            device = prev_device
-            device.name = local_name
+            prev_details: dict[str, Any] = service_info.device.details
             prev_details.update(details)
+            # _rssi is deprecated, will be removed in newer bleak
             # pylint: disable-next=protected-access
-            device._rssi = rssi  # deprecated, will be removed in newer bleak
+            service_info.device._rssi = rssi
+            has_local_name = bool(local_name)
+            if prev_name and (not has_local_name or len(prev_name) > len(local_name)):  # type: ignore[arg-type]
+                service_info.name = prev_name
+            else:
+                service_info.device.name = local_name
+                service_info.name = local_name if has_local_name else address  # type: ignore[assignment]
+
+            has_service_uuids = bool(service_uuids)
+            if has_service_uuids and service_uuids != prev_service_info.service_uuids:
+                service_info.service_uuids = list(
+                    {*service_uuids, *prev_service_info.service_uuids}
+                )
+            elif not has_service_uuids:
+                service_info.service_uuids = prev_service_info.service_uuids
+            else:
+                service_info.service_uuids = service_uuids
+
+            has_service_data = bool(service_data)
+            if has_service_data and service_data != prev_service_info.service_data:
+                service_info.service_data = {
+                    **prev_service_info.service_data,
+                    **service_data,
+                }
+            elif not has_service_data:
+                service_info.service_data = prev_service_info.service_data
+            else:
+                service_info.service_data = service_data
+
+            has_manufacturer_data = bool(manufacturer_data)
+            if (
+                has_manufacturer_data
+                and manufacturer_data != prev_service_info.manufacturer_data
+            ):
+                service_info.manufacturer_data = {
+                    **prev_service_info.manufacturer_data,
+                    **manufacturer_data,
+                }
+            elif not has_manufacturer_data:
+                service_info.manufacturer_data = prev_service_info.manufacturer_data
+            else:
+                service_info.manufacturer_data = manufacturer_data
 
-        service_info = BluetoothServiceInfoBleak.__new__(BluetoothServiceInfoBleak)
-        service_info.name = local_name or address
         service_info.address = address
         service_info.rssi = rssi
-        service_info.manufacturer_data = manufacturer_data
-        service_info.service_data = service_data
-        service_info.service_uuids = service_uuids
         service_info.source = self.source
-        service_info.device = device
         service_info._advertisement = None
         service_info.connectable = self.connectable
         service_info.time = advertisement_monotonic_time
