@@ -3,7 +3,7 @@
 import time
 from datetime import timedelta
 from typing import Any
-from unittest.mock import patch
+from unittest.mock import ANY, patch
 
 import pytest
 from bleak_retry_connector import AllocationChange, Allocations, BleakSlotManager
@@ -13,6 +13,8 @@ from freezegun import freeze_time
 from habluetooth import (
     BluetoothManager,
     HaBluetoothSlotAllocations,
+    HaScannerRegistration,
+    HaScannerRegistrationEvent,
     get_manager,
     set_manager,
 )
@@ -24,7 +26,7 @@ from . import (
     inject_advertisement_with_source,
     utcnow,
 )
-from .conftest import FakeBluetoothAdapters
+from .conftest import FakeBluetoothAdapters, FakeScanner
 
 
 @pytest.mark.asyncio
@@ -265,7 +267,14 @@ async def test_async_register_allocation_callback(
         switchbot_device_signal_100, switchbot_adv_signal_100, "hci1"
     )
 
-    assert manager.async_current_allocations() == []
+    assert manager.async_current_allocations() == [
+        HaBluetoothSlotAllocations(
+            source="AA:BB:CC:DD:EE:00", slots=5, free=5, allocated=[]
+        ),
+        HaBluetoothSlotAllocations(
+            source="AA:BB:CC:DD:EE:11", slots=5, free=5, allocated=[]
+        ),
+    ]
     manager.async_on_allocation_changed(
         Allocations(
             "AA:BB:CC:DD:EE:00",
@@ -307,10 +316,13 @@ async def test_async_register_allocation_callback(
     assert len(ok_allocations) == 2
 
     assert manager.async_current_allocations() == [
-        HaBluetoothSlotAllocations("AA:BB:CC:DD:EE:00", 5, 4, ["44:44:33:11:23:12"])
+        HaBluetoothSlotAllocations("AA:BB:CC:DD:EE:00", 5, 4, ["44:44:33:11:23:12"]),
+        HaBluetoothSlotAllocations(
+            source="AA:BB:CC:DD:EE:11", slots=5, free=5, allocated=[]
+        ),
     ]
     assert manager.async_current_allocations("AA:BB:CC:DD:EE:00") == [
-        HaBluetoothSlotAllocations("AA:BB:CC:DD:EE:00", 5, 4, ["44:44:33:11:23:12"])
+        HaBluetoothSlotAllocations("AA:BB:CC:DD:EE:00", 5, 4, ["44:44:33:11:23:12"]),
     ]
     cancel1()
     cancel2()
@@ -332,3 +344,127 @@ async def test_async_register_allocation_callback_non_connectable(
             allocated=[],
         ),
     ]
+
+
+@pytest.mark.asyncio
+@pytest.mark.usefixtures("enable_bluetooth")
+async def test_async_register_scanner_registration_callback(
+    register_hci0_scanner: None,
+    register_hci1_scanner: None,
+) -> None:
+    """Test bluetooth async_register_scanner_registration_callback handles failures."""
+    manager = get_manager()
+    assert manager._loop is not None
+
+    scanners = manager.async_current_scanners()
+    assert len(scanners) == 2
+    sources = {scanner.source for scanner in scanners}
+    assert sources == {"AA:BB:CC:DD:EE:00", "AA:BB:CC:DD:EE:11"}
+
+    failed_scanner_callbacks: list[HaScannerRegistration] = []
+
+    def _failing_callback(scanner_registration: HaScannerRegistration) -> None:
+        """Failing callback."""
+        failed_scanner_callbacks.append(scanner_registration)
+        raise ValueError("This is a test")
+
+    ok_scanner_callbacks: list[HaScannerRegistration] = []
+
+    def _ok_callback(scanner_registration: HaScannerRegistration) -> None:
+        """Ok callback."""
+        ok_scanner_callbacks.append(scanner_registration)
+
+    cancel1 = manager.async_register_scanner_registration_callback(
+        _failing_callback, None
+    )
+    # Make sure the second callback still works if the first one fails and
+    # raises an exception
+    cancel2 = manager.async_register_scanner_registration_callback(_ok_callback, None)
+
+    hci3_scanner = FakeScanner("AA:BB:CC:DD:EE:33", "hci3")
+    hci3_scanner.connectable = True
+    manager = get_manager()
+    cancel = manager.async_register_scanner(hci3_scanner, connection_slots=5)
+
+    assert len(ok_scanner_callbacks) == 1
+    assert ok_scanner_callbacks[0] == HaScannerRegistration(
+        HaScannerRegistrationEvent.ADDED, hci3_scanner
+    )
+    assert len(failed_scanner_callbacks) == 1
+
+    cancel()
+
+    assert len(ok_scanner_callbacks) == 2
+    assert ok_scanner_callbacks[1] == HaScannerRegistration(
+        HaScannerRegistrationEvent.REMOVED, hci3_scanner
+    )
+    cancel1()
+    cancel2()
+
+
+@pytest.mark.asyncio
+async def test_async_register_scanner_with_connection_slots() -> None:
+    """Test registering a scanner with connection slots."""
+    manager = get_manager()
+    assert manager._loop is not None
+
+    scanners = manager.async_current_scanners()
+    assert len(scanners) == 0
+
+    hci3_scanner = FakeScanner("AA:BB:CC:DD:EE:33", "hci3")
+    hci3_scanner.connectable = True
+    manager = get_manager()
+    cancel = manager.async_register_scanner(hci3_scanner, connection_slots=5)
+    assert manager.async_current_allocations(hci3_scanner.source) == [
+        HaBluetoothSlotAllocations(hci3_scanner.source, 5, 5, [])
+    ]
+
+    cancel()
+
+
+@pytest.mark.asyncio
+@pytest.mark.usefixtures("enable_bluetooth")
+async def test_diagnostics(register_hci0_scanner: None) -> None:
+    """Test bluetooth diagnostics."""
+    manager = get_manager()
+    assert manager._loop is not None
+    manager.async_on_allocation_changed(
+        Allocations(
+            "AA:BB:CC:DD:EE:00",
+            5,
+            4,
+            ["44:44:33:11:23:12"],
+        )
+    )
+    diagnostics = await manager.async_diagnostics()
+    assert diagnostics == {
+        "adapters": {},
+        "advertisement_tracker": ANY,
+        "all_history": ANY,
+        "allocations": {
+            "AA:BB:CC:DD:EE:00": {
+                "allocated": ["44:44:33:11:23:12"],
+                "free": 4,
+                "slots": 5,
+                "source": "AA:BB:CC:DD:EE:00",
+            }
+        },
+        "connectable_history": ANY,
+        "scanners": [
+            {
+                "discovered_devices_and_advertisement_data": [],
+                "last_detection": 0.0,
+                "monotonic_time": ANY,
+                "name": "hci0 (AA:BB:CC:DD:EE:00)",
+                "scanning": True,
+                "source": "AA:BB:CC:DD:EE:00",
+                "start_time": 0.0,
+                "type": "FakeScanner",
+            }
+        ],
+        "slot_manager": {
+            "adapter_slots": {"hci0": 5},
+            "allocations_by_adapter": {"hci0": []},
+            "manager": False,
+        },
+    }
